@@ -1,4 +1,6 @@
-# -*- coding: utf-8 -*-
+# Copyright (c) Jupyter Development Team.
+# Distributed under the terms of the Modified BSD License.
+
 """
 This module is meant to run JupyterLab in a headless browser, making sure
 the application launches and starts up without errors.
@@ -41,14 +43,14 @@ class LogErrorHandler(logging.Handler):
         super().__init__(level=logging.ERROR)
         self.errored = False
 
-    def filter(self, record):
+    def filter(self, record):  # noqa
         # Handle known StreamClosedError from Tornado
         # These occur when we forcibly close Websockets or
         # browser connections during the test.
         # https://github.com/tornadoweb/tornado/issues/2834
         if (
             hasattr(record, "exc_info")
-            and not record.exc_info is None
+            and record.exc_info is not None
             and isinstance(record.exc_info[1], (StreamClosedError, WebSocketClosedError))
         ):
             return
@@ -142,8 +144,8 @@ async def run_browser(url):
     if not osp.exists(osp.join(target, "node_modules")):
         if not osp.exists(target):
             os.makedirs(osp.join(target))
-        await run_async_process(["jlpm", "init", "-y"], cwd=target)
-        await run_async_process(["jlpm", "add", "playwright@^1.9.2"], cwd=target)
+        await run_async_process(["npm", "init", "-y"], cwd=target)
+        await run_async_process(["npm", "install", "playwright@^1.9.2"], cwd=target)
     shutil.copy(osp.join(here, "browser-test.js"), osp.join(target, "browser-test.js"))
     await run_async_process(["node", "browser-test.js", url], cwd=target)
 
@@ -153,10 +155,10 @@ def run_browser_sync(url):
     target = osp.join(get_app_dir(), "browser_test")
     if not osp.exists(osp.join(target, "node_modules")):
         os.makedirs(target)
-        subprocess.call(["jlpm", "init", "-y"], cwd=target)
-        subprocess.call(["jlpm", "add", "playwright@^1.9.2"], cwd=target)
+        subprocess.call(["npm", "init", "-y"], cwd=target)  # noqa S603 S607
+        subprocess.call(["npm", "install", "playwright@^1.9.2"], cwd=target)  # noqa S603 S607
     shutil.copy(osp.join(here, "browser-test.js"), osp.join(target, "browser-test.js"))
-    return subprocess.check_call(["node", "browser-test.js", url], cwd=target)
+    return subprocess.check_call(["node", "browser-test.js", url], cwd=target)  # noqa S603 S607
 
 
 class BrowserApp(LabApp):
@@ -175,7 +177,7 @@ class BrowserApp(LabApp):
     test_browser = Bool(True)
 
     def initialize_settings(self):
-        self.settings.setdefault("page_config_data", dict())
+        self.settings.setdefault("page_config_data", {})
         self.settings["page_config_data"]["browserTest"] = True
         self.settings["page_config_data"]["buildAvailable"] = False
         self.settings["page_config_data"]["exposeAppInBrowser"] = True
@@ -193,18 +195,6 @@ def _jupyter_server_extension_points():
     return [{"module": __name__, "app": BrowserApp}]
 
 
-# TODO: remove handling of --notebook arg and the following two
-# functions in JupyterLab 4.0
-def load_jupyter_server_extension(serverapp):
-    serverapp.log.info("Loading BrowserApp as a classic notebook (v6) extension.")
-    extension = BrowserApp()
-    extension.serverapp = serverapp
-    extension.load_config_file()
-    extension.update_config(serverapp.config)
-    extension.parse_command_line(serverapp.extra_args)
-    extension.initialize()
-
-
 def _jupyter_server_extension_paths():
     return [{"module": "jupyterlab.browser_check"}]
 
@@ -216,13 +206,4 @@ if __name__ == "__main__":
             BrowserApp.test_browser = False
             sys.argv.remove(option)
 
-    if "--notebook" in sys.argv:
-        from notebook.notebookapp import NotebookApp
-
-        NotebookApp.default_url = "/lab"
-        sys.argv.remove("--notebook")
-        NotebookApp.nbserver_extensions = {"jupyterlab.browser_check": True}
-        NotebookApp.open_browser = False
-        NotebookApp.launch_instance()
-    else:
         BrowserApp.launch_instance()
