@@ -1,10 +1,8 @@
-# coding: utf-8
 """Utilities for installing Javascript extensions for the notebook"""
 
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-from __future__ import print_function
 
 import importlib
 import json
@@ -16,9 +14,13 @@ import subprocess
 import sys
 from pathlib import Path
 
-from os.path import basename
+try:
+    from importlib.metadata import PackageNotFoundError, version
+except ImportError:
+    from importlib_metadata import PackageNotFoundError, version
+
+from os.path import basename, normpath
 from os.path import join as pjoin
-from os.path import normpath
 
 from jupyter_core.paths import ENV_JUPYTER_PATH, SYSTEM_JUPYTER_PATH, jupyter_data_dir
 from jupyter_core.utils import ensure_dir_exists
@@ -42,7 +44,7 @@ HERE = osp.abspath(osp.dirname(__file__))
 # ------------------------------------------------------------------------------
 
 
-def develop_labextension(
+def develop_labextension(  # noqa
     path,
     symlink=True,
     overwrite=False,
@@ -92,9 +94,8 @@ def develop_labextension(
     ensure_dir_exists(labext)
 
     if isinstance(path, (list, tuple)):
-        raise TypeError(
-            "path must be a string pointing to a single extension to install; call this function multiple times to install multiple extensions"
-        )
+        msg = "path must be a string pointing to a single extension to install; call this function multiple times to install multiple extensions"
+        raise TypeError(msg)
 
     if not destination:
         destination = basename(normpath(path))
@@ -115,16 +116,17 @@ def develop_labextension(
         path = os.path.abspath(path)
         if not os.path.exists(full_dest):
             if logger:
-                logger.info("Symlinking: %s -> %s" % (full_dest, path))
+                logger.info(f"Symlinking: {full_dest} -> {path}")
             try:
                 os.symlink(path, full_dest)
             except OSError as e:
                 if platform.platform().startswith("Windows"):
-                    raise OSError(
+                    msg = (
                         "Symlinks can be activated on Windows 10 for Python version 3.8 or higher"
                         " by activating the 'Developer Mode'. That may not be allowed by your administrators.\n"
                         "See https://docs.microsoft.com/en-us/windows/apps/get-started/enable-your-device-for-development"
-                    ) from e
+                    )
+                    raise OSError(msg) from e
                 raise
 
         elif not os.path.islink(full_dest):
@@ -132,7 +134,7 @@ def develop_labextension(
 
     elif os.path.isdir(path):
         path = pjoin(os.path.abspath(path), "")  # end in path separator
-        for parent, dirs, files in os.walk(path):
+        for parent, _, files in os.walk(path):
             dest_dir = pjoin(full_dest, parent[len(path) :])
             if not os.path.exists(dest_dir):
                 if logger:
@@ -172,7 +174,7 @@ def develop_labextension_py(
         src = os.path.join(base_path, labext["src"])
         dest = labext["dest"]
         if logger:
-            logger.info("Installing %s -> %s" % (src, dest))
+            logger.info(f"Installing {src} -> {dest}")
 
         if not os.path.exists(src):
             build_labextension(base_path, logger=logger)
@@ -196,12 +198,9 @@ def build_labextension(
     path, logger=None, development=False, static_url=None, source_map=False, core_path=None
 ):
     """Build a labextension in the given path"""
-    if core_path is None:
-        core_path = osp.join(HERE, "staging")
-    else:
-        core_path = str(Path(core_path).resolve())
+    core_path = osp.join(HERE, "staging") if core_path is None else str(Path(core_path).resolve())
 
-    ext_path = osp.abspath(path)
+    ext_path = str(Path(path).resolve())
 
     if logger:
         logger.info("Building extension in %s" % path)
@@ -216,18 +215,15 @@ def build_labextension(
     if source_map:
         arguments.append("--source-map")
 
-    subprocess.check_call(arguments, cwd=ext_path)
+    subprocess.check_call(arguments, cwd=ext_path)  # noqa S603
 
 
 def watch_labextension(
     path, labextensions_path, logger=None, development=False, source_map=False, core_path=None
 ):
     """Watch a labextension in a given path"""
-    if core_path is None:
-        core_path = osp.join(HERE, "staging")
-    else:
-        core_path = str(Path(core_path).resolve())
-    ext_path = osp.abspath(path)
+    core_path = osp.join(HERE, "staging") if core_path is None else str(Path(core_path).resolve())
+    ext_path = str(Path(path).resolve())
 
     if logger:
         logger.info("Building extension in %s" % path)
@@ -254,7 +250,7 @@ def watch_labextension(
     if source_map:
         arguments.append("--source-map")
 
-    subprocess.check_call(arguments, cwd=ext_path)
+    subprocess.check_call(arguments, cwd=ext_path)  # noqa S603
 
 
 # ------------------------------------------------------------------------------
@@ -269,45 +265,46 @@ def _ensure_builder(ext_path, core_path):
         core_data = json.load(fid)
     with open(osp.join(ext_path, "package.json")) as fid:
         ext_data = json.load(fid)
-    depVersion1 = core_data["devDependencies"]["@jupyterlab/builder"]
-    depVersion2 = ext_data.get("devDependencies", dict()).get("@jupyterlab/builder")
-    depVersion2 = depVersion2 or ext_data.get("dependencies", dict()).get("@jupyterlab/builder")
-    if depVersion2 is None:
+    dep_version1 = core_data["devDependencies"]["@jupyterlab/builder"]
+    dep_version2 = ext_data.get("devDependencies", {}).get("@jupyterlab/builder")
+    dep_version2 = dep_version2 or ext_data.get("dependencies", {}).get("@jupyterlab/builder")
+    if dep_version2 is None:
         raise ValueError(
-            "Extensions require a devDependency on @jupyterlab/builder@%s" % depVersion1
+            "Extensions require a devDependency on @jupyterlab/builder@%s" % dep_version1
         )
 
     # if we have installed from disk (version is a path), assume we know what
     # we are doing and do not check versions.
-    if "/" in depVersion2:
-        with open(osp.join(ext_path, depVersion2, "package.json")) as fid:
-            depVersion2 = json.load(fid).get("version")
+    if "/" in dep_version2:
+        with open(osp.join(ext_path, dep_version2, "package.json")) as fid:
+            dep_version2 = json.load(fid).get("version")
     if not osp.exists(osp.join(ext_path, "node_modules")):
-        subprocess.check_call(["jlpm"], cwd=ext_path)
+        subprocess.check_call(["jlpm"], cwd=ext_path)  # noqa S603 S607
 
     # Find @jupyterlab/builder using node module resolution
     # We cannot use a script because the script path is a shell script on Windows
     target = ext_path
     while not osp.exists(osp.join(target, "node_modules", "@jupyterlab", "builder")):
         if osp.dirname(target) == target:
-            raise ValueError("Could not find @jupyterlab/builder")
+            msg = "Could not find @jupyterlab/builder"
+            raise ValueError(msg)
         target = osp.dirname(target)
 
-    overlap = _test_overlap(depVersion1, depVersion2, drop_prerelease1=True, drop_prerelease2=True)
+    overlap = _test_overlap(
+        dep_version1, dep_version2, drop_prerelease1=True, drop_prerelease2=True
+    )
     if not overlap:
         with open(
             osp.join(target, "node_modules", "@jupyterlab", "builder", "package.json")
         ) as fid:
-            depVersion2 = json.load(fid).get("version")
+            dep_version2 = json.load(fid).get("version")
         overlap = _test_overlap(
-            depVersion1, depVersion2, drop_prerelease1=True, drop_prerelease2=True
+            dep_version1, dep_version2, drop_prerelease1=True, drop_prerelease2=True
         )
 
     if not overlap:
-        raise ValueError(
-            "Extensions require a devDependency on @jupyterlab/builder@%s, you have a dependency on %s"
-            % (depVersion1, depVersion2)
-        )
+        msg = f"Extensions require a devDependency on @jupyterlab/builder@{dep_version1}, you have a dependency on {dep_version2}"
+        raise ValueError(msg)
 
     return osp.join(
         target, "node_modules", "@jupyterlab", "builder", "lib", "build-labextension.js"
@@ -331,11 +328,11 @@ def _should_copy(src, dest, logger=None):
     """
     if not os.path.exists(dest):
         return True
-    if os.stat(src).st_mtime - os.stat(dest).st_mtime > 1e-6:
+    if os.stat(src).st_mtime - os.stat(dest).st_mtime > 1e-6:  # noqa
         # we add a fudge factor to work around a bug in python 2.x
         # that was fixed in python 3.x: https://bugs.python.org/issue12904
         if logger:
-            logger.warn("Out of date: %s" % dest)
+            logger.warning("Out of date: %s" % dest)
         return True
     if logger:
         logger.info("Up to date: %s" % dest)
@@ -357,7 +354,7 @@ def _maybe_copy(src, dest, logger=None):
     """
     if _should_copy(src, dest, logger=logger):
         if logger:
-            logger.info("Copying: %s -> %s" % (src, dest))
+            logger.info(f"Copying: {src} -> {dest}")
         shutil.copy2(src, dest)
 
 
@@ -382,13 +379,12 @@ def _get_labextension_dir(user=False, sys_prefix=False, prefix=None, labextensio
         ("labextensions_dir", labextensions_dir),
         ("sys_prefix", sys_prefix),
     ]
-    conflicting_set = ["{}={!r}".format(n, v) for n, v in conflicting if v]
+    conflicting_set = [f"{n}={v!r}" for n, v in conflicting if v]
     if len(conflicting_set) > 1:
-        raise ArgumentConflict(
-            "cannot specify more than one of user, sys_prefix, prefix, or labextensions_dir, but got: {}".format(
-                ", ".join(conflicting_set)
-            )
+        msg = "cannot specify more than one of user, sys_prefix, prefix, or labextensions_dir, but got: {}".format(
+            ", ".join(conflicting_set)
         )
+        raise ArgumentConflict(msg)
     if user:
         labext = pjoin(jupyter_data_dir(), "labextensions")
     elif sys_prefix:
@@ -402,7 +398,7 @@ def _get_labextension_dir(user=False, sys_prefix=False, prefix=None, labextensio
     return labext
 
 
-def _get_labextension_metadata(module):
+def _get_labextension_metadata(module):  # noqa
     """Get the list of labextension paths associated with a Python module.
 
     Returns a tuple of (the module path,             [{
@@ -419,7 +415,8 @@ def _get_labextension_metadata(module):
     """
     mod_path = osp.abspath(module)
     if not osp.exists(mod_path):
-        raise FileNotFoundError("The path `{}` does not exist.".format(mod_path))
+        msg = f"The path `{mod_path}` does not exist."
+        raise FileNotFoundError(msg)
 
     errors = []
 
@@ -431,7 +428,7 @@ def _get_labextension_metadata(module):
     except Exception as exc:
         errors.append(exc)
 
-     # Try to get the package name
+    # Try to get the package name
     package = None
 
     # Try getting the package name from pyproject.toml
@@ -440,27 +437,28 @@ def _get_labextension_metadata(module):
             data = load(fid)
         package = data.get("project", {}).get("name")
 
-    # Try getting the package name from setup.py invocation.
+    # Try getting the package name from setup.py
     if not package:
         try:
             package = (
-                subprocess.check_output([sys.executable, "setup.py", "--name"], cwd=mod_path)
+                subprocess.check_output(
+                    [sys.executable, "setup.py", "--name"], cwd=mod_path  # noqa S603
+                )
                 .decode("utf8")
                 .strip()
             )
         except subprocess.CalledProcessError:
-            raise FileNotFoundError(
+            msg = (
                 "The Python package `{}` is not a valid package, "
                 "it is missing the `setup.py` file.".format(module)
             )
+            raise FileNotFoundError(msg) from None
 
     # Make sure the package is installed
-    import pkg_resources
-
     try:
-        pkg_resources.get_distribution(package)
-    except pkg_resources.DistributionNotFound:
-        subprocess.check_call([sys.executable, "-m", "pip", "install", "-e", mod_path])
+        version(package)
+    except PackageNotFoundError:
+        subprocess.check_call([sys.executable, "-m", "pip", "install", "-e", mod_path])  # noqa S603
         sys.path.insert(0, mod_path)
 
     from setuptools import find_namespace_packages, find_packages
@@ -481,6 +479,5 @@ def _get_labextension_metadata(module):
         except Exception as exc:
             errors.append(exc)
 
-    raise ModuleNotFoundError(
-        "There is no labextension at {}. Errors encountered: {}".format(module, errors)
-    )
+    msg = f"There is no labextension at {module}. Errors encountered: {errors}"
+    raise ModuleNotFoundError(msg)
